class UICheatSheet:
    CONCISE_INFO = {
        'wsb': 'White Status Bar',
        'bsb': 'Black Status Bar',
        'sf': 'Scaffold',
        'sfa': 'Scaffold\'s appbar',
        'sfb': 'Scaffold\'s body',
        'ct': 'Container',
        'sb': 'SizedBox',
        'c': 'Column',
        'r': 'Row',
        'w': 'Wrap',
        's': 'Stack',
        'sv': 'Single Child Scroll View',
        'lv': 'List View',
        'lvb': 'List View builder',
        'gv': 'Grid View',
        'gvb': 'Grid View builder',
        'pw': 'Paging Widget',
        'i': 'Image View',
        't': 'Text View',
        'tf': 'Text Field',
        'b': 'Button',
        'bb': 'Bloc Builder',
        'bl': 'Bloc Listener',
        'bp': 'Bloc Provider',
        'mbl': 'Multi Bloc Listener',
        'mbp': 'Multi Bloc Provider',
        'stf': 'Stateful Widget, default is Stateless for custom Widget',
        'dt': 'Default Toolbar',
        'da': 'Default Appbar',
        'wp': 'Custom Web Page',
        'gp': 'Custom Gallery Page',
        'g': 'Gallery View'
    }
