export 'package:easy_localization/easy_localization.dart';
export 'package:flutter_screenutil/flutter_screenutil.dart';
export 'package:{{ package_name }}/generated/assets.gen.dart';
export 'package:{{ package_name }}/generated/colors.gen.dart';
export 'package:{{ package_name }}/generated/fonts.gen.dart';
export 'package:{{ package_name }}/generated/locale_keys.g.dart';
